/* required signel replacement for SYSV to enable reliable signals.
   since this works with BSD and it posix, we'll just use it instead of 
   signal
*/

/* SIGSTOP and SIGKILL cannot be trapped so they are not here. 
*/

#include "includes.h"

Sigfunc * falc_signal(int signo, Sigfunc *func)
{
  struct sigaction act, oact;

  act.sa_handler = func;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;

  if(signo == SIGALRM)
  {
#ifdef SA_INTERRUPT
    act.sa_flags |= SA_INTERRUPT; /* SUNOS */    
#endif
  }
  else 
  {
#ifdef SA_RESTART /* this is where we tell it to restart signals */
    act.sa_flags |= SA_RESTART; /* SVR4 & 4.3BSD */
#endif
  }

  if(sigaction(signo, &act, &oact) < 0)
    return(SIG_ERR);
   
  return(oact.sa_handler);
}

void catch_signals(void)
{
  extern int debug;

  if(falc_signal(SIGABRT, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGABRT");

  if(falc_signal(SIGFPE, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGFPE");

  if(falc_signal(SIGCONT, sig_cont) == SIG_ERR)
    DB perror("Can't catch SIGCONT");

  if(falc_signal(SIGHUP, sig_hup) == SIG_ERR)
    DB perror("Can't catch SIGHUP");

  if(falc_signal(SIGILL, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGILL");

  if(falc_signal(SIGINT, sig_int) == SIG_ERR)
    DB perror("Can't catch SIGINT");

  if(falc_signal(SIGPIPE, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGPIPE");

  if(falc_signal(SIGQUIT, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGQUIT");

  if(falc_signal(SIGSEGV, sig_segv) == SIG_ERR)
    DB perror("Can't catch SIGSEGV");

  if(falc_signal(SIGTERM, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGTERM");

  if(falc_signal(SIGTTIN, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGTTIN");

  if(falc_signal(SIGTTOU, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGTTOU");

  if(falc_signal(SIGUSR1, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGUSR1");

  if(falc_signal(SIGUSR2, sig_default) == SIG_ERR)
    DB perror("Can't catch SIGUSR2");

  if(falc_signal(SIGCHLD, sig_chld) == SIG_ERR)
    DB perror("Can't catch SIGCHLD");

  return;
}

static void sig_cont(int signo) /* after a ^Z, reset the term */
{
  extern int debug;

  DB printf("caught SIGCONT\n");
}

static void sig_chld(int signo) /* catch the child dying. */
{
  pid_t pid;
  int stat;
  extern int debug;
  
  DB printf("Caught SIGCHLD\n");

  while((pid = waitpid(-1, &stat, WNOHANG)) > 0);

  return;
}

static void sig_hup(int signo) /* this is for furture use */
{
  extern int debug;
  DB printf("caught SIGHUP, reloading configs.\n");
  
/*  reload_config();
*/

  return;
}

static void sig_int(int signo) /* catch ^C and quit with grace */
{
  extern int debug;

  DB printf("caught SIGINT\n");
  exit();
}

/* catch a SEGV, tell you about it, cleanup and quit */
static void sig_segv(int signo) 
{
  printf("caught a SEGV!  Check for core!");
}

static void sig_default(int signo) /* catch all, cleanup and quit */
{
  extern int debug;
  DB printf("something bad has occured!  caught singal %d\n", signo);
}

